@@ -170,6 +170,7 @@ gcm_dbus_get_profiles_for_device_internal (GcmDbus *dbus, const gchar *sysfs_pat
 	guint i;
 	GcmDevice *device;
 	GcmProfile *profile;
+	GFile *file;
 	GError *error = NULL;
 	GPtrArray *array;
 	GPtrArray *array_devices;
@@ -184,7 +185,7 @@ gcm_dbus_get_profiles_for_device_internal (GcmDbus *dbus, const gchar *sysfs_pat
 
 		/* get the native path of this device */
 		g_object_get (device,
-			      "native-device-sysfs", &sysfs_path_tmp,
+			      "native-device", &sysfs_path_tmp,
 			      NULL);
 
 		/* wrong type of device */
@@ -200,7 +201,8 @@ gcm_dbus_get_profiles_for_device_internal (GcmDbus *dbus, const gchar *sysfs_pat
 
 			/* open and parse filename */
 			profile = gcm_profile_default_new ();
-			ret = gcm_profile_parse (profile, filename, &error);
+			file = g_file_new_for_path (filename);
+			ret = gcm_profile_parse (profile, file, &error);
 			if (!ret) {
 				egg_warning ("failed to parse %s: %s", filename, error->message);
 				g_clear_error (&error);
@@ -209,6 +211,7 @@ gcm_dbus_get_profiles_for_device_internal (GcmDbus *dbus, const gchar *sysfs_pat
 			}
 
 			/* unref */
+			g_object_unref (file);
 			g_object_unref (profile);
 			g_free (filename);
 		}
@@ -498,6 +501,7 @@ gcm_dbus_init (GcmDbus *dbus)
 	dbus->priv = GCM_DBUS_GET_PRIVATE (dbus);
 	dbus->priv->gconf_client = gconf_client_get_default ();
 	dbus->priv->client = gcm_client_new ();
+	gcm_client_set_use_threads (dbus->priv->client, TRUE);
 	dbus->priv->profile_store = gcm_profile_store_new ();
 	dbus->priv->timer = g_timer_new ();
 
