@@ -29,10 +29,7 @@
 #include "config.h"
 
 #include <glib-object.h>
-#include <math.h>
-#include <gio/gio.h>
 #include <gconf/gconf-client.h>
-#include <libgnomeui/gnome-rr.h>
 
 #include "gcm-device.h"
 #include "gcm-profile.h"
@@ -52,6 +49,7 @@ static void     gcm_device_finalize	(GObject     *object);
 struct _GcmDevicePrivate
 {
 	gboolean			 connected;
+	gboolean			 virtual;
 	gboolean			 saved;
 	gfloat				 gamma;
 	gfloat				 brightness;
@@ -64,8 +62,7 @@ struct _GcmDevicePrivate
 	gchar				*profile_filename;
 	gchar				*title;
 	GConfClient			*gconf_client;
-	gchar				*native_device_xrandr;
-	gchar				*native_device_sysfs;
+	GcmColorspaceEnum		 colorspace;
 };
 
 enum {
@@ -73,6 +70,7 @@ enum {
 	PROP_TYPE,
 	PROP_ID,
 	PROP_CONNECTED,
+	PROP_VIRTUAL,
 	PROP_SAVED,
 	PROP_SERIAL,
 	PROP_MODEL,
@@ -82,8 +80,7 @@ enum {
 	PROP_CONTRAST,
 	PROP_PROFILE_FILENAME,
 	PROP_TITLE,
-	PROP_NATIVE_DEVICE_XRANDR,
-	PROP_NATIVE_DEVICE_SYSFS,
+	PROP_COLORSPACE,
 	PROP_LAST
 };
 
@@ -203,7 +200,7 @@ gcm_device_load (GcmDevice *device, GError **error)
 	ret = g_key_file_has_group (file, device->priv->id);
 	if (!ret) {
 		/* not fatal */
-		egg_warning ("failed to find parameters for %s", device->priv->id);
+		egg_debug ("failed to find saved parameters for %s", device->priv->id);
 		ret = TRUE;
 		goto out;
 	}
@@ -362,6 +359,13 @@ gcm_device_save (GcmDevice *device, GError **error)
 		g_key_file_set_string (keyfile, device->priv->id, "title", device->priv->title);
 	g_key_file_set_string (keyfile, device->priv->id, "type", gcm_device_type_enum_to_string (device->priv->type));
 
+	/* add colorspace */
+	g_key_file_set_string (keyfile, device->priv->id, "colorspace", gcm_colorspace_enum_to_string (device->priv->colorspace));
+
+	/* add virtual */
+	if (device->priv->virtual)
+		g_key_file_set_boolean (keyfile, device->priv->id, "virtual", TRUE);
+
 	/* convert to string */
 	data = g_key_file_to_data (keyfile, NULL, &error_local);
 	if (data == NULL) {
@@ -394,6 +398,27 @@ out:
 	return ret;
 }
 
+/**
+ * gcm_device_apply:
+ **/
+gboolean
+gcm_device_apply (GcmDevice *device, GError **error)
+{
+	gboolean ret = TRUE;
+	GcmDeviceClass *klass = GCM_DEVICE_GET_CLASS (device);
+
+	/* no support */
+	if (klass->apply == NULL) {
+		egg_debug ("no klass support for %s", device->priv->id);
+		goto out;
+	}
+
+	/* run the callback */
+	ret = klass->apply (device, error);
+out:
+	return ret;
+}
+
 /**
  * gcm_device_get_property:
  **/
@@ -413,6 +438,9 @@ gcm_device_get_property (GObject *object, guint prop_id, GValue *value, GParamSp
 	case PROP_CONNECTED:
 		g_value_set_boolean (value, priv->connected);
 		break;
+	case PROP_VIRTUAL:
+		g_value_set_boolean (value, priv->virtual);
+		break;
 	case PROP_SAVED:
 		g_value_set_boolean (value, priv->saved);
 		break;
@@ -440,11 +468,8 @@ gcm_device_get_property (GObject *object, guint prop_id, GValue *value, GParamSp
 	case PROP_TITLE:
 		g_value_set_string (value, priv->title);
 		break;
-	case PROP_NATIVE_DEVICE_XRANDR:
-		g_value_set_string (value, priv->native_device_xrandr);
-		break;
-	case PROP_NATIVE_DEVICE_SYSFS:
-		g_value_set_string (value, priv->native_device_sysfs);
+	case PROP_COLORSPACE:
+		g_value_set_uint (value, priv->colorspace);
 		break;
 	default:
 		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
@@ -472,6 +497,9 @@ gcm_device_set_property (GObject *object, guint prop_id, const GValue *value, GP
 	case PROP_CONNECTED:
 		priv->connected = g_value_get_boolean (value);
 		break;
+	case PROP_VIRTUAL:
+		priv->virtual = g_value_get_boolean (value);
+		break;
 	case PROP_SAVED:
 		priv->saved = g_value_get_boolean (value);
 		break;
@@ -504,13 +532,8 @@ gcm_device_set_property (GObject *object, guint prop_id, const GValue *value, GP
 	case PROP_CONTRAST:
 		priv->contrast = g_value_get_float (value);
 		break;
-	case PROP_NATIVE_DEVICE_XRANDR:
-		g_free (priv->native_device_xrandr);
-		priv->native_device_xrandr = g_strdup (g_value_get_string (value));
-		break;
-	case PROP_NATIVE_DEVICE_SYSFS:
-		g_free (priv->native_device_sysfs);
-		priv->native_device_sysfs = g_strdup (g_value_get_string (value));
+	case PROP_COLORSPACE:
+		priv->colorspace = g_value_get_uint (value);
 		break;
 	default:
 		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
@@ -554,6 +577,14 @@ gcm_device_class_init (GcmDeviceClass *klass)
 				      G_PARAM_READWRITE);
 	g_object_class_install_property (object_class, PROP_CONNECTED, pspec);
 
+	/**
+	 * GcmDevice:virtual:
+	 */
+	pspec = g_param_spec_boolean ("virtual", NULL, NULL,
+				      FALSE,
+				      G_PARAM_READWRITE);
+	g_object_class_install_property (object_class, PROP_VIRTUAL, pspec);
+
 	/**
 	 * GcmDevice:saved:
 	 */
@@ -627,20 +658,12 @@ gcm_device_class_init (GcmDeviceClass *klass)
 	g_object_class_install_property (object_class, PROP_TITLE, pspec);
 
 	/**
-	 * GcmDevice:native-device-xrandr:
+	 * GcmDevice:colorspace:
 	 */
-	pspec = g_param_spec_string ("native-device-xrandr", NULL, NULL,
-				     NULL,
-				     G_PARAM_READWRITE);
-	g_object_class_install_property (object_class, PROP_NATIVE_DEVICE_XRANDR, pspec);
-
-	/**
-	 * GcmDevice:native-device-sysfs:
-	 */
-	pspec = g_param_spec_string ("native-device-sysfs", NULL, NULL,
-				     NULL,
-				     G_PARAM_READWRITE);
-	g_object_class_install_property (object_class, PROP_NATIVE_DEVICE_SYSFS, pspec);
+	pspec = g_param_spec_uint ("colorspace", NULL, NULL,
+				   0, GCM_COLORSPACE_ENUM_LAST, GCM_COLORSPACE_ENUM_UNKNOWN,
+				   G_PARAM_READWRITE);
+	g_object_class_install_property (object_class, PROP_COLORSPACE, pspec);
 
 	g_type_class_add_private (klass, sizeof (GcmDevicePrivate));
 }
@@ -656,11 +679,10 @@ gcm_device_init (GcmDevice *device)
 	device->priv->id = NULL;
 	device->priv->saved = FALSE;
 	device->priv->connected = FALSE;
+	device->priv->virtual = FALSE;
 	device->priv->serial = NULL;
 	device->priv->manufacturer = NULL;
 	device->priv->model = NULL;
-	device->priv->native_device_xrandr = NULL;
-	device->priv->native_device_sysfs = NULL;
 	device->priv->profile_filename = NULL;
 	device->priv->gconf_client = gconf_client_get_default ();
 	device->priv->gamma = gconf_client_get_float (device->priv->gconf_client, GCM_SETTINGS_DEFAULT_GAMMA, &error);
@@ -672,6 +694,7 @@ gcm_device_init (GcmDevice *device)
 		device->priv->gamma = 1.0f;
 	device->priv->brightness = 0.0f;
 	device->priv->contrast = 100.f;
+	device->priv->colorspace = GCM_COLORSPACE_ENUM_UNKNOWN;
 }
 
 /**
@@ -688,32 +711,20 @@ gcm_device_finalize (GObject *object)
 	g_free (priv->serial);
 	g_free (priv->manufacturer);
 	g_free (priv->model);
-	g_free (priv->native_device_xrandr);
-	g_free (priv->native_device_sysfs);
+	g_free (priv->profile_filename);
 	g_object_unref (priv->gconf_client);
 
 	G_OBJECT_CLASS (gcm_device_parent_class)->finalize (object);
 }
 
-/**
- * gcm_device_new:
- *
- * Return value: a new GcmDevice object.
- **/
-GcmDevice *
-gcm_device_new (void)
-{
-	GcmDevice *device;
-	device = g_object_new (GCM_TYPE_DEVICE, NULL);
-	return GCM_DEVICE (device);
-}
-
 /***************************************************************************
  ***                          MAKE CHECK TESTS                           ***
  ***************************************************************************/
 #ifdef EGG_TEST
 #include "egg-test.h"
 
+#include "gcm-device-udev.h"
+
 void
 gcm_device_test (EggTest *test)
 {
@@ -731,7 +742,7 @@ gcm_device_test (EggTest *test)
 
 	/************************************************************/
 	egg_test_title (test, "get a device object");
-	device = gcm_device_new ();
+	device = gcm_device_udev_new ();
 	egg_test_assert (test, device != NULL);
 
 	/************************************************************/
@@ -765,7 +776,9 @@ gcm_device_test (EggTest *test)
 		      "type", GCM_DEVICE_TYPE_ENUM_SCANNER,
 		      "id", "sysfs_dummy_device",
 		      "connected", TRUE,
+		      "virtual", FALSE,
 		      "serial", "0123456789",
+		      "colorspace", GCM_COLORSPACE_ENUM_RGB,
 		      NULL);
 
 	/************************************************************/
@@ -859,7 +872,7 @@ gcm_device_test (EggTest *test)
 
 	/************************************************************/
 	egg_test_title (test, "check data");
-	if (g_strcmp0 (data, "\n[sysfs_dummy_device]\nprofile=/srv/sysfs_canon_canoscan.icc\nserial=0123456789\ntype=scanner\n") == 0)
+	if (g_strcmp0 (data, "\n[sysfs_dummy_device]\nprofile=/srv/sysfs_canon_canoscan.icc\nserial=0123456789\ntype=scanner\ncolorspace=rgb\n") == 0)
 		egg_test_success (test, NULL);
 	else
 		egg_test_failed (test, "invalid data: %s", data);
