@@ -23,17 +23,10 @@
 
 #include <glib/gi18n.h>
 #include <gtk/gtk.h>
-#include <gdk/gdkx.h>
-#include <libgnomeui/gnome-rr.h>
-#include <X11/extensions/Xrandr.h>
-#include <X11/extensions/xf86vmode.h>
-#include <gconf/gconf-client.h>
 #include <dbus/dbus-glib.h>
+#include <gdk/gdkx.h>
 
 #include "gcm-utils.h"
-#include "gcm-screen.h"
-#include "gcm-clut.h"
-#include "gcm-xserver.h"
 
 #include "egg-debug.h"
 
@@ -83,16 +76,18 @@ gcm_utils_linkify (const gchar *text)
  * gcm_utils_is_icc_profile:
  **/
 gboolean
-gcm_utils_is_icc_profile (const gchar *filename)
+gcm_utils_is_icc_profile (GFile *file)
 {
-	GFile *file;
 	GFileInfo *info;
 	const gchar *type;
 	GError *error = NULL;
 	gboolean ret = FALSE;
+	gchar *filename = NULL;
 
 	/* get content type for file */
-	file = g_file_new_for_path (filename);
+	filename = g_file_get_uri (file);
+	if (filename == NULL)
+		filename = g_file_get_path (file);
 	info = g_file_query_info (file, G_FILE_ATTRIBUTE_STANDARD_CONTENT_TYPE, G_FILE_QUERY_INFO_NONE, NULL, &error);
 	if (info != NULL) {
 		type = g_file_info_get_attribute_string (info, G_FILE_ATTRIBUTE_STANDARD_CONTENT_TYPE);
@@ -125,7 +120,7 @@ gcm_utils_is_icc_profile (const gchar *filename)
 out:
 	if (info != NULL)
 		g_object_unref (info);
-	g_object_unref (file);
+	g_free (filename);
 	return ret;
 }
 
@@ -165,6 +160,9 @@ gcm_utils_install_package (const gchar *package_name, GtkWindow *window)
 					   "/org/freedesktop/PackageKit",
 					   "org.freedesktop.PackageKit.Modify");
 
+	/* set timeout */
+	dbus_g_proxy_set_default_timeout (proxy, G_MAXINT);
+
 	/* execute sync method */
 	ret = dbus_g_proxy_call (proxy, "InstallPackageNames", &error,
 				 G_TYPE_UINT, xid,
@@ -242,310 +240,66 @@ gcm_utils_ensure_printable (gchar *text)
 }
 
 /**
- * gcm_utils_get_gamma_size_fallback:
+ * gcm_utils_mkdir_with_parents:
  **/
-static guint
-gcm_utils_get_gamma_size_fallback (void)
+gboolean
+gcm_utils_mkdir_with_parents (const gchar *filename, GError **error)
 {
-	guint size;
-	Bool rc;
+	gboolean ret;
+	GFile *file = NULL;
 
-	/* this is per-screen, not per output which is less than ideal */
-	gdk_error_trap_push ();
-	egg_warning ("using PER-SCREEN gamma tables as driver is not XRANDR 1.3 compliant");
-	rc = XF86VidModeGetGammaRampSize (GDK_DISPLAY(), gdk_x11_get_default_screen (), (int*) &size);
-	gdk_error_trap_pop ();
-	if (!rc)
-		size = 0;
-
-	return size;
-}
-
-/**
- * gcm_utils_get_gamma_size:
- *
- * Return value: the gamma size, or 0 if error;
- **/
-static guint
-gcm_utils_get_gamma_size (GnomeRRCrtc *crtc, GError **error)
-{
-	guint id;
-	guint size;
-
-	/* get id that X recognises */
-	id = gnome_rr_crtc_get_id (crtc);
-
-	/* get the value, and catch errors */
-	gdk_error_trap_push ();
-	size = XRRGetCrtcGammaSize (GDK_DISPLAY(), id);
-	if (gdk_error_trap_pop ())
-		size = 0;
-
-	/* some drivers support Xrandr 1.2, not 1.3 */
-	if (size == 0)
-		size = gcm_utils_get_gamma_size_fallback ();
-
-	/* no size, or X popped an error */
-	if (size == 0) {
-		g_set_error_literal (error, 1, 0, "failed to get gamma size");
-		goto out;
-	}
-out:
-	return size;
-}
-
-/**
- * gcm_utils_set_gamma_fallback:
- **/
-static gboolean
-gcm_utils_set_gamma_fallback (XRRCrtcGamma *gamma, guint size)
-{
-	Bool rc;
-
-	/* this is per-screen, not per output which is less than ideal */
-	gdk_error_trap_push ();
-	egg_warning ("using PER-SCREEN gamma tables as driver is not XRANDR 1.3 compliant");
-	rc = XF86VidModeSetGammaRamp (GDK_DISPLAY(), gdk_x11_get_default_screen (), size, gamma->red, gamma->green, gamma->blue);
-	gdk_error_trap_pop ();
-
-	return rc;
-}
-
-/**
- * gcm_utils_set_gamma_for_crtc:
- *
- * Return value: %TRUE for success;
- **/
-static gboolean
-gcm_utils_set_gamma_for_crtc (GnomeRRCrtc *crtc, GcmClut *clut, GError **error)
-{
-	guint id;
-	gboolean ret = TRUE;
-	GPtrArray *array = NULL;
-	XRRCrtcGamma *gamma = NULL;
-	guint i;
-	GcmClutData *data;
-
-	/* get data */
-	array = gcm_clut_get_array (clut);
-	if (array == NULL) {
-		ret = FALSE;
-		g_set_error_literal (error, 1, 0, "failed to get CLUT data");
-		goto out;
-	}
-
-	/* no length? */
-	if (array->len == 0) {
-		ret = FALSE;
-		g_set_error_literal (error, 1, 0, "no data in the CLUT array");
-		goto out;
-	}
-
-	/* convert to a type X understands */
-	gamma = XRRAllocGamma (array->len);
-	for (i=0; i<array->len; i++) {
-		data = g_ptr_array_index (array, i);
-		gamma->red[i] = data->red;
-		gamma->green[i] = data->green;
-		gamma->blue[i] = data->blue;
-	}
-
-	/* get id that X recognises */
-	id = gnome_rr_crtc_get_id (crtc);
-
-	/* get the value, and catch errors */
-	gdk_error_trap_push ();
-	XRRSetCrtcGamma (GDK_DISPLAY(), id, gamma);
-	gdk_flush ();
-	if (gdk_error_trap_pop ()) {
-		/* some drivers support Xrandr 1.2, not 1.3 */
-		ret = gcm_utils_set_gamma_fallback (gamma, array->len);
-		if (!ret) {
-			g_set_error (error, 1, 0, "failed to set crtc gamma %p (%i) on %i", gamma, array->len, id);
+	/* ensure desination exists */
+	ret = g_file_test (filename, G_FILE_TEST_EXISTS);
+	if (!ret) {
+		file = g_file_new_for_path (filename);
+		ret = g_file_make_directory_with_parents (file, NULL, error);
+		if (!ret)
 			goto out;
-		}
 	}
 out:
-	if (gamma != NULL)
-		XRRFreeGamma (gamma);
-	if (array != NULL)
-		g_ptr_array_unref (array);
+	if (file != NULL)
+		g_object_unref (file);
 	return ret;
 }
 
 /**
- * gcm_utils_set_gamma_for_device:
- *
- * Return value: %TRUE for success;
+ * gcm_utils_mkdir_for_filename:
  **/
 gboolean
-gcm_utils_set_gamma_for_device (GcmDevice *device, GError **error)
+gcm_utils_mkdir_for_filename (const gchar *filename, GError **error)
 {
 	gboolean ret = FALSE;
-	GcmClut *clut = NULL;
-	GcmProfile *profile = NULL;
-	GcmXserver *xserver = NULL;
-	GnomeRRCrtc *crtc;
-	GnomeRROutput *output;
-	gint x, y;
-	gchar *filename = NULL;
-	gchar *filename_systemwide = NULL;
-	gfloat gamma;
-	gfloat brightness;
-	gfloat contrast;
-	gchar *output_name;
-	gchar *id = NULL;
-	guint size;
-	gboolean saved;
-	gboolean use_global;
-	gboolean use_atom;
-	gboolean leftmost_screen = FALSE;
-	GcmDeviceTypeEnum type;
-	GcmScreen *screen = NULL;
-	GConfClient *gconf_client = NULL;
+	GFile *file;
+	GFile *parent_dir = NULL;
 
-	g_return_val_if_fail (device != NULL, FALSE);
-
-	/* use gconf to decide to set LUT or set ATOMs */
-	gconf_client = gconf_client_get_default ();
-
-	/* get details about the device */
-	g_object_get (device,
-		      "id", &id,
-		      "type", &type,
-		      "saved", &saved,
-		      "profile-filename", &filename,
-		      "gamma", &gamma,
-		      "brightness", &brightness,
-		      "contrast", &contrast,
-		      "native-device-xrandr", &output_name,
-		      NULL);
-
-	/* do no set the gamma for non-display types */
-	if (type != GCM_DEVICE_TYPE_ENUM_DISPLAY) {
-		g_set_error (error, 1, 0, "not a display: %s", id);
+	/* get a file from the URI / path */
+	file = g_file_new_for_uri (filename);
+	if (file == NULL)
+		file = g_file_new_for_path (filename);
+	if (file == NULL) {
+		g_set_error (error, 1, 0, "could not resolve file for %s", filename);
 		goto out;
 	}
 
-	/* should be set for display types */
-	if (output_name == NULL) {
-		g_set_error (error, 1, 0, "no output name for display: %s", id);
+	/* get parent */
+	parent_dir = g_file_get_parent (file);
+	if (parent_dir == NULL) {
+		g_set_error (error, 1, 0, "could not get parent dir %s", filename);
 		goto out;
 	}
 
-	/* if not saved, try to find default filename */
-	if (!saved && filename == NULL) {
-		filename_systemwide = g_strdup_printf ("%s/%s.icc", GCM_SYSTEM_PROFILES_DIR, id);
-		ret = g_file_test (filename_systemwide, G_FILE_TEST_EXISTS);
-		if (ret) {
-			egg_error ("using systemwide %s as profile", filename_systemwide);
-			filename = g_strdup (filename_systemwide);
-		}
-	}
-
-	/* check we have an output */
-	screen = gcm_screen_new ();
-	output = gcm_screen_get_output_by_name (screen, output_name, error);
-	if (output == NULL)
-		goto out;
-
-	/* get crtc size */
-	crtc = gnome_rr_output_get_crtc (output);
-	if (crtc == NULL) {
-		g_set_error (error, 1, 0, "failed to get crtc for device: %s", id);
-		goto out;
-	}
-
-	/* get gamma table size */
-	size = gcm_utils_get_gamma_size (crtc, error);
-	if (size == 0)
-		goto out;
-
-	/* only set the CLUT if we're not seting the atom */
-	use_global = gconf_client_get_bool (gconf_client, GCM_SETTINGS_GLOBAL_DISPLAY_CORRECTION, NULL);
-	if (use_global && filename != NULL) {
-		/* create CLUT */
-		profile = gcm_profile_default_new ();
-		ret = gcm_profile_parse (profile, filename, error);
+	/* ensure desination exists */
+	ret = g_file_query_exists (parent_dir, NULL);
+	if (!ret) {
+		ret = g_file_make_directory_with_parents (parent_dir, NULL, error);
 		if (!ret)
 			goto out;
-
-		/* create a CLUT from the profile */
-		clut = gcm_profile_generate_vcgt (profile, size);
-	}
-
-	/* create dummy CLUT if we failed */
-	if (clut == NULL) {
-		clut = gcm_clut_new ();
-		g_object_set (clut,
-			      "size", size,
-			      NULL);
-	}
-
-	/* do fine adjustment */
-	if (use_global) {
-		g_object_set (clut,
-			      "gamma", gamma,
-			      "brightness", brightness,
-			      "contrast", contrast,
-			      NULL);
-	}
-
-	/* actually set the gamma */
-	ret = gcm_utils_set_gamma_for_crtc (crtc, clut, error);
-	if (!ret)
-		goto out;
-
-	/* set per output and per-screen atoms */
-	xserver = gcm_xserver_new ();
-
-	/* is the monitor our primary monitor */
-	gnome_rr_output_get_position (output, &x, &y);
-	leftmost_screen = (x == 0 && y == 0);
-
-	/* either remove the atoms or set them */
-	use_atom = gconf_client_get_bool (gconf_client, GCM_SETTINGS_SET_ICC_PROFILE_ATOM, NULL);
-	if (!use_atom || filename == NULL) {
-
-		/* remove the output atom if there's nothing to show */
-		ret = gcm_xserver_remove_output_profile (xserver, output_name, error);
-		if (!ret)
-			goto out;
-
-		/* primary screen */
-		if (leftmost_screen) {
-			ret = gcm_xserver_remove_root_window_profile (xserver, error);
-			if (!ret)
-				goto out;
-		}
-	} else {
-		/* set the per-output and per screen profile atoms */
-		ret = gcm_xserver_set_output_profile (xserver, output_name, filename, error);
-		if (!ret)
-			goto out;
-
-		/* primary screen */
-		if (leftmost_screen) {
-			ret = gcm_xserver_set_root_window_profile (xserver, filename, error);
-			if (!ret)
-				goto out;
-		}
 	}
 out:
-	g_free (id);
-	g_free (filename);
-	g_free (filename_systemwide);
-	g_free (output_name);
-	if (gconf_client != NULL)
-		g_object_unref (gconf_client);
-	if (screen != NULL)
-		g_object_unref (screen);
-	if (clut != NULL)
-		g_object_unref (clut);
-	if (profile != NULL)
-		g_object_unref (profile);
-	if (xserver != NULL)
-		g_object_unref (xserver);
+	if (file != NULL)
+		g_object_unref (file);
+	if (parent_dir != NULL)
+		g_object_unref (parent_dir);
 	return ret;
 }
 
@@ -553,61 +307,53 @@ out:
  * gcm_utils_mkdir_and_copy:
  **/
 gboolean
-gcm_utils_mkdir_and_copy (const gchar *source, const gchar *destination, GError **error)
+gcm_utils_mkdir_and_copy (GFile *source, GFile *destination, GError **error)
 {
 	gboolean ret;
-	gchar *path;
-	GFile *sourcefile;
-	GFile *destfile;
-	GFile *destpath;
+	GFile *parent;
 
 	g_return_val_if_fail (source != NULL, FALSE);
 	g_return_val_if_fail (destination != NULL, FALSE);
 
-	/* setup paths */
-	sourcefile = g_file_new_for_path (source);
-	path = g_path_get_dirname (destination);
-	destpath = g_file_new_for_path (path);
-	destfile = g_file_new_for_path (destination);
+	/* get parent */
+	parent = g_file_get_parent (destination);
 
-	/* ensure desination exists */
-	ret = g_file_test (path, G_FILE_TEST_EXISTS);
-	if (!ret) {
-		ret = g_file_make_directory_with_parents  (destpath, NULL, error);
+	/* create directory */
+	if (!g_file_query_exists (parent, NULL)) {
+		ret = g_file_make_directory_with_parents (parent, NULL, error);
 		if (!ret)
 			goto out;
 	}
 
 	/* do the copy */
-	egg_debug ("copying from %s to %s", source, path);
-	ret = g_file_copy (sourcefile, destfile, G_FILE_COPY_OVERWRITE, NULL, NULL, NULL, error);
+	ret = g_file_copy (source, destination, G_FILE_COPY_OVERWRITE, NULL, NULL, NULL, error);
 	if (!ret)
 		goto out;
 out:
-	g_free (path);
-	g_object_unref (sourcefile);
-	g_object_unref (destpath);
-	g_object_unref (destfile);
+	g_object_unref (parent);
 	return ret;
 }
 
 /**
  * gcm_utils_get_profile_destination:
  **/
-gchar *
-gcm_utils_get_profile_destination (const gchar *filename)
+GFile *
+gcm_utils_get_profile_destination (GFile *file)
 {
 	gchar *basename;
 	gchar *destination;
+	GFile *dest;
 
-	g_return_val_if_fail (filename != NULL, NULL);
+	g_return_val_if_fail (file != NULL, NULL);
 
 	/* get destination filename for this source file */
-	basename = g_path_get_basename (filename);
+	basename = g_file_get_basename (file);
 	destination = g_build_filename (g_get_home_dir (), GCM_PROFILE_PATH, basename, NULL);
+	dest = g_file_new_for_path (destination);
 
 	g_free (basename);
-	return destination;
+	g_free (destination);
+	return dest;
 }
 
 /**
@@ -811,6 +557,8 @@ gcm_utils_test (EggTest *test)
 	gchar *filename;
 	GcmProfileTypeEnum profile_type;
 	GcmDeviceTypeEnum device_type;
+	GFile *file;
+	GFile *dest;
 
 	if (!egg_test_start (test, "GcmUtils"))
 		return;
@@ -828,18 +576,24 @@ gcm_utils_test (EggTest *test)
 
 	/************************************************************/
 	egg_test_title (test, "get filename of data file");
-	filename = gcm_utils_get_profile_destination ("dave.icc");
+	file = g_file_new_for_path ("dave.icc");
+	dest = gcm_utils_get_profile_destination (file);
+	filename = g_file_get_path (dest);
 	if (g_str_has_suffix (filename, "/.color/icc/dave.icc"))
 		egg_test_success (test, NULL);
 	else
 		egg_test_failed (test, "failed to get filename: %s", filename);
 	g_free (filename);
+	g_object_unref (file);
+	g_object_unref (dest);
 
 	/************************************************************/
 	egg_test_title (test, "check is icc profile");
 	filename = egg_test_get_data_file ("bluish.icc");
-	ret = gcm_utils_is_icc_profile (filename);
+	file = g_file_new_for_path (filename);
+	ret = gcm_utils_is_icc_profile (file);
 	egg_test_assert (test, ret);
+	g_object_unref (file);
 	g_free (filename);
 
 	/************************************************************/
